from __future__ import print_function

import sys

sys.path.insert(0, '../src')
sys.path.insert(0, '../graph_methods')
sys.path.insert(0, '../graph_methods_regression')

import argparse
import pandas as pd
import csv
import numpy as np
import json
import sys
import random
from dataloader import *
from function import *
import time


def run_single_deep_regression():
    from deep_regression import SingleRegression

    with open(config_json_file, 'r') as f:
        conf = json.load(f)
    label_name_list = [target_name]
    print('label_name_list ', label_name_list)

    # read data
    test_index = [running_index]
    train_index = filter(lambda x: x not in test_index, np.arange(5))
    train_file_list = file_list[train_index]
    test_file_list = file_list[test_index]
    print('train files ', train_file_list)
    print('test files ', test_file_list)

    train_dataset = FingerprintsDataSet(train_file_list,
                                        feature_name='Fingerprints',
                                        label_name_list=[target_name])
    test_dataset = FingerprintsDataSet(test_file_list,
                                       feature_name='Fingerprints',
                                       label_name_list=[target_name])
    print('done data preparation')

    kwargs = {'train_dataset': train_dataset, 'test_dataset': test_dataset}
    task = SingleRegression(conf=conf, **kwargs)
    task.train_and_predict(weight_file)

    task.eval_with_existing(weight_file)
    y_test, y_pred_on_test = task.make_prediction(task.test_dataloader)
    np.savez('output_on_test', y_test=y_test, y_pred=y_pred_on_test)
    return


def run_single_deep_regression_flatten_random_projection():
    from deep_regression import SingleRegression

    with open(config_json_file, 'r') as f:
        conf = json.load(f)
    conf['input_dimension'] = n_gram_num * random_projection_dimension * segmentation_num
    label_name_list = [target_name]
    print('label_name_list ', label_name_list)

    # read data
    test_index = [running_index]
    train_index = filter(lambda x: x not in test_index, np.arange(5))
    train_file_list = file_list[train_index]
    test_file_list = file_list[test_index]
    print('train files ', train_file_list)
    print('test files ', test_file_list)

    train_dataset = Fingerprints_Random_Projection_DataSet(train_file_list,
                                                           feature_name='random_projected_list',
                                                           label_name_list=['label_name'],
                                                           n_gram_num=n_gram_num,
                                                           is_sum_up_feature_segment=is_sum_up_feature_segment)
    test_dataset = Fingerprints_Random_Projection_DataSet(test_file_list,
                                                          feature_name='random_projected_list',
                                                          label_name_list=['label_name'],
                                                          n_gram_num=n_gram_num,
                                                          is_sum_up_feature_segment=is_sum_up_feature_segment)
    print('done data preparation')

    kwargs = {'train_dataset': train_dataset, 'test_dataset': test_dataset}
    task = SingleRegression(conf=conf, **kwargs)
    task.train_and_predict(weight_file)

    task.eval_with_existing(weight_file)
    y_test, y_pred_on_test = task.make_prediction(task.test_dataset)
    np.savez('output_on_test', y_test=y_test, y_pred=y_pred_on_test)
    return


def run_xgboost_regression():
    from xgboost_regression import XGBoostRegression

    with open(config_json_file, 'r') as f:
        conf = json.load(f)
    label_name_list = [target_name]
    print('label_name_list ', label_name_list)

    # read data
    test_index = [running_index]
    train_index = filter(lambda x: x not in test_index, np.arange(5))
    train_file_list = file_list[train_index]
    test_file_list = file_list[test_index]
    print('train files ', train_file_list)
    print('test files ', test_file_list)

    train_pd = filter_out_missing_values(read_merged_data(train_file_list), label_list=label_name_list)
    test_pd = filter_out_missing_values(read_merged_data(test_file_list), label_list=label_name_list)

    X_train, y_train = extract_feature_and_label(train_pd,
                                                 feature_name='Fingerprints',
                                                 label_name_list=label_name_list)
    X_test, y_test = extract_feature_and_label(test_pd,
                                               feature_name='Fingerprints',
                                               label_name_list=label_name_list)
    print('done data preparation')

    task = XGBoostRegression(conf=conf)
    task.train_and_predict(X_train, y_train, X_test, y_test, weight_file)

    task.eval_with_existing(X_train, y_train, X_test, y_test, weight_file)
    y_pred_on_test = task.predict_with_existing(X_test, weight_file)
    np.savez('output_on_test', y_test=y_test, y_pred=y_pred_on_test)
    return


def run_xgboost_regression_flatten_random_projection():
    from xgboost_regression import XGBoostRegression

    with open(config_json_file, 'r') as f:
        conf = json.load(f)
    label_name_list = [target_name]
    print('label_name_list ', label_name_list)

    # read data
    test_index = [running_index]
    train_index = filter(lambda x: x not in test_index, np.arange(5))
    train_file_list = file_list[train_index]
    test_file_list = file_list[test_index]
    print('train files ', train_file_list)
    print('test files ', test_file_list)

    X_train, y_train = extract_feature_and_label_npy(train_file_list,
                                                     feature_name='random_projected_list',
                                                     label_name_list=label_name_list,
                                                     n_gram_num=n_gram_num,
                                                     is_sum_up_feature_segment=is_sum_up_feature_segment)
    X_test, y_test = extract_feature_and_label_npy(test_file_list,
                                                   feature_name='random_projected_list',
                                                   label_name_list=label_name_list,
                                                   n_gram_num=n_gram_num,
                                                   is_sum_up_feature_segment=is_sum_up_feature_segment)
    print('done data preparation')

    task = XGBoostRegression(conf=conf)
    task.train_and_predict(X_train, y_train, X_test, y_test, weight_file)

    task.eval_with_existing(X_train, y_train, X_test, y_test, weight_file)
    y_pred_on_test = task.predict_with_existing(X_test, weight_file)
    np.savez('output_on_test', y_test=y_test, y_pred=y_pred_on_test)
    return


def run_random_forest_regression():
    from random_forest_regression import RandomForestRegression

    with open(config_json_file, 'r') as f:
        conf = json.load(f)
    label_name_list = [target_name]
    print('label_name_list ', label_name_list)

    # read data
    test_index = [running_index]
    train_index = filter(lambda x: x not in test_index, np.arange(5))
    train_file_list = file_list[train_index]
    test_file_list = file_list[test_index]
    print('train files ', train_file_list)
    print('test files ', test_file_list)

    train_pd = filter_out_missing_values(read_merged_data(train_file_list), label_list=label_name_list)
    test_pd = filter_out_missing_values(read_merged_data(test_file_list), label_list=label_name_list)

    X_train, y_train = extract_feature_and_label(train_pd,
                                                 feature_name='Fingerprints',
                                                 label_name_list=label_name_list)
    X_test, y_test = extract_feature_and_label(test_pd,
                                               feature_name='Fingerprints',
                                               label_name_list=label_name_list)
    print('done data preparation')

    task = RandomForestRegression(conf=conf)
    task.train_and_predict(X_train, y_train, X_test, y_test, weight_file)

    task.eval_with_existing(X_train, y_train, X_test, y_test, weight_file)
    y_pred_on_test = task.predict_with_existing(X_test, weight_file)
    np.savez('output_on_test', y_test=y_test, y_pred=y_pred_on_test)
    return


def run_random_forest_regression_flatten_random_projection():
    from random_forest_regression import RandomForestRegression

    with open(config_json_file, 'r') as f:
        conf = json.load(f)
    label_name_list = [target_name]
    print('label_name_list ', label_name_list)

    # read data
    test_index = [running_index]
    train_index = filter(lambda x: x not in test_index, np.arange(5))
    train_file_list = file_list[train_index]
    test_file_list = file_list[test_index]
    print('train files ', train_file_list)
    print('test files ', test_file_list)

    X_train, y_train = extract_feature_and_label_npy(train_file_list,
                                                     feature_name='random_projected_list',
                                                     label_name_list=label_name_list,
                                                     n_gram_num=n_gram_num,
                                                     is_sum_up_feature_segment=is_sum_up_feature_segment)
    X_test, y_test = extract_feature_and_label_npy(test_file_list,
                                                   feature_name='random_projected_list',
                                                   label_name_list=label_name_list,
                                                   n_gram_num=n_gram_num,
                                                   is_sum_up_feature_segment=is_sum_up_feature_segment)
    print('done data preparation')

    task = RandomForestRegression(conf=conf)
    task.train_and_predict(X_train, y_train, X_test, y_test, weight_file)

    task.eval_with_existing(X_train, y_train, X_test, y_test, weight_file)
    y_pred_on_test = task.predict_with_existing(X_test, weight_file)
    np.savez('output_on_test', y_test=y_test, y_pred=y_pred_on_test)
    return


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--config_json_file', required=True)
    parser.add_argument('--weight_file', required=True)
    parser.add_argument('--model',  required=True)
    parser.add_argument('--target_name', default='E1-CC2')
    parser.add_argument('--n_gram_num', type=int, default=6)
    parser.add_argument('--random_projection_dimension', type=int, default=100)
    parser.add_argument('--running_index', type=int, default=0)
    given_args = parser.parse_args()

    config_json_file = given_args.config_json_file
    weight_file = given_args.weight_file
    n_gram_num = given_args.n_gram_num
    random_projection_dimension = given_args.random_projection_dimension
    running_index = given_args.running_index
    is_sum_up_feature_segment = False

    K = 5
    target_name = given_args.target_name
    model = given_args.model

    if 'n_gram' in model:
        directory = '../node2vec/datasets/{}/{}/{{}}_grammed_cbow_{}_graph.npz'.format('qm8', running_index, random_projection_dimension)
    else:
        directory = '../datasets/{}/{{}}.csv.gz'.format('qm8')
    file_list = []
    for i in range(K):
        file_list.append(directory.format(i))
    file_list = np.array(file_list)

    start_time = time.time()
    if model == 'morgan_dnn':
        run_single_deep_regression()
    elif model == 'n_gram_dnn':
        run_single_deep_regression_flatten_random_projection()
    elif model == 'morgan_xgb':
        run_xgboost_regression()
    elif model == 'n_gram_xgb':
        run_xgboost_regression_flatten_random_projection()
    elif model == 'morgan_rf':
        run_random_forest_regression()
    elif model == 'n_gram_rf':
        run_random_forest_regression_flatten_random_projection()
    else:
        raise Exception('No such model! Should be among [{}, {}, {}, {}, {}, {}].'.format(
            'morgan_dnn',
            'n_gram_dnn',
            'morgan_xgb',
            'n_gram_xgb',
            'morgan_rf',
            'n_gram_rf'
        ))
    end_time = time.time()
    print('Running time: {}'.format(end_time - start_time))

    import os
    os.rename('output_on_test.npz', '../output/{}/{}/{}.npz'.format(running_index, model, target_name))
